@@ -33,28 +33,29 @@ int search_binary_handler(struct linux_binprm *bprm, struct pt_regs *regs);
 
 #ifdef CONFIG_CCSECURITY
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
-/* Obtain prototype of __d_path() or d_absolute_path(). */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) && LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
+/* Obtain prototype of __d_path(). */
 #include <linux/dcache.h>
 #endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-/* Obtain prototype of find_task_by_vpid() and find_task_by_pid_ns(). */
-#include <linux/sched.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+/* Obtain definition of kuid_t and kgid_t. */
+#include <linux/uidgid.h>
 #endif
 
 /* For exporting variables and functions. */
 struct ccsecurity_exports {
 	void (*load_policy) (const char *filename);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0)
-	typeof(d_absolute_path) (*d_absolute_path);
+	char * (*d_absolute_path) (const struct path *, char *, int);
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
 	typeof(__d_path) (*__d_path);
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 	spinlock_t *vfsmount_lock;
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-	typeof(find_task_by_vpid) (*find_task_by_vpid);
-	typeof(find_task_by_pid_ns) (*find_task_by_pid_ns);
+	struct task_struct * (*find_task_by_vpid) (pid_t nr);
+	struct task_struct * (*find_task_by_pid_ns) (pid_t nr,
+						     struct pid_namespace *ns);
 #endif
 };
 
@@ -65,14 +66,14 @@ struct ccsecurity_operations {
 	int (*chroot_permission) (struct path *path);
 	int (*pivot_root_permission) (struct path *old_path,
 				      struct path *new_path);
-	int (*mount_permission) (char *dev_name, struct path *path,
+	int (*mount_permission) (const char *dev_name, struct path *path,
 				 const char *type, unsigned long flags,
 				 void *data_page);
 #else
 	int (*chroot_permission) (struct nameidata *nd);
 	int (*pivot_root_permission) (struct nameidata *old_nd,
 				      struct nameidata *new_nd);
-	int (*mount_permission) (char *dev_name, struct nameidata *nd,
+	int (*mount_permission) (const char *dev_name, struct nameidata *nd,
 				 const char *type, unsigned long flags,
 				 void *data_page);
 #endif
@@ -170,14 +171,13 @@ static inline int ccs_pivot_root_permission(struct path *old_path,
 	return func ? func(old_path, new_path) : 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct path *path,
-				       char *type, unsigned long flags,
+static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+				       const char *type, unsigned long flags,
 				       void *data_page)
 {
-	int (*func) (char *, struct path *, const char *, unsigned long,
+	int (*func) (const char *, struct path *, const char *, unsigned long,
 		     void *) = ccsecurity_ops.mount_permission;
-	return func ? func(dev_name, path, (const char *) type, flags,
-			   data_page) : 0;
+	return func ? func(dev_name, path, type, flags, data_page) : 0;
 }
 
 #else
@@ -196,14 +196,13 @@ static inline int ccs_pivot_root_permission(struct nameidata *old_nd,
 	return func ? func(old_nd, new_nd) : 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct nameidata *nd,
-				       char *type, unsigned long flags,
-				       void *data_page)
+static inline int ccs_mount_permission(const char *dev_name,
+				       struct nameidata *nd, const char *type,
+				       unsigned long flags, void *data_page)
 {
-	int (*func) (char *, struct nameidata *, const char *, unsigned long,
-		     void *) = ccsecurity_ops.mount_permission;
-	return func ? func(dev_name, nd, (const char *) type, flags,
-			   data_page) : 0;
+	int (*func) (const char *, struct nameidata *, const char *,
+		     unsigned long, void *) = ccsecurity_ops.mount_permission;
+	return func ? func(dev_name, nd, type, flags, data_page) : 0;
 }
 
 #endif
@@ -415,8 +414,8 @@ static inline int ccs_pivot_root_permission(struct path *old_path,
 	return 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct path *path,
-				       char *type, unsigned long flags,
+static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+				       const char *type, unsigned long flags,
 				       void *data_page)
 {
 	return 0;
@@ -435,9 +434,9 @@ static inline int ccs_pivot_root_permission(struct nameidata *old_nd,
 	return 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct nameidata *nd,
-				       char *type, unsigned long flags,
-				       void *data_page)
+static inline int ccs_mount_permission(const char *dev_name,
+				       struct nameidata *nd, const char *type,
+				       unsigned long flags, void *data_page)
 {
 	return 0;
 }
